( cd usr/lib ; rm -rf libgiftproto.so )
( cd usr/lib ; ln -sf libgiftproto.so.0.0.0 libgiftproto.so )
( cd usr/lib ; rm -rf libgiftproto.so.0 )
( cd usr/lib ; ln -sf libgiftproto.so.0.0.0 libgiftproto.so.0 )
( cd usr/lib ; rm -rf libgift.so.0 )
( cd usr/lib ; ln -sf libgift.so.0.0.0 libgift.so.0 )
( cd usr/lib ; rm -rf libgift.so )
( cd usr/lib ; ln -sf libgift.so.0.0.0 libgift.so )
